module BotProg where

import System
import Random
import System.Posix.Process

type BotName = String
type AllBots = [(BotName, Bool -> IO ())]

data Options = Options {
    opt_bot_name :: String,
    opt_help :: Bool,
    opt_verbose :: Bool,
    opt_seed :: Maybe String
}

init_options :: BotName -> Options
init_options default_bot_name = Options {
    opt_bot_name = default_bot_name,
    opt_help = False,
    opt_verbose = False,
    opt_seed = Nothing
}

bot_prog :: AllBots -> BotName -> IO ()
bot_prog all_bots default_bot_name =
    do let usage = get_usage all_bots default_bot_name
           options = init_options default_bot_name
       args <- getArgs
       bot_prog_parse_args args usage all_bots options

bot_prog_parse_args ::
    [String] -> IO () -> AllBots -> Options -> IO ()
bot_prog_parse_args args usage all_bots options =
    case args of
        [] -> bot_prog_with_options usage all_bots options
        (arg:rest) | arg == "-h" || arg == "--help" -> 
            bot_prog_parse_args rest usage all_bots
            options { opt_help = True }
        (arg:rest) | arg == "-v" || arg == "--verbose" -> 
            bot_prog_parse_args rest usage all_bots
            options { opt_verbose = True }
        (arg:seed:rest) | arg == "-s" || arg == "--seed" -> 
            bot_prog_parse_args rest usage all_bots
            options { opt_seed = Just seed }
        (arg@('-':_):_) -> do putStrLn ("ERROR: bad argument: " ++ arg)
                              usage
        (bot_name:rest) ->
            bot_prog_parse_args rest usage all_bots
            options { opt_bot_name = bot_name }

bot_prog_with_options :: IO () -> AllBots -> Options -> IO ()
bot_prog_with_options usage all_bots options =
    do init_std_gen (opt_seed options)
       if (opt_help options)
         then usage
         else do let bot_name = (opt_bot_name options)
                 case lookup bot_name all_bots of
                   Just bot_action -> bot_action (opt_verbose options)
                   Nothing -> do putStrLn ("ERROR: bad bot name: " ++ bot_name)
                                 usage

init_std_gen :: Maybe String -> IO ()
init_std_gen Nothing =
    do random_int <- getStdRandom random
       pid <- getProcessID
       let random_integer = toInteger (random_int :: Integer)
           pid_integer = toInteger pid
           seed_integer = random_integer + pid_integer
           seed_int = fromInteger seed_integer
       setStdGen (mkStdGen seed_int)
init_std_gen (Just seed) =
    setStdGen (read seed)

get_usage :: AllBots -> BotName -> IO ()
get_usage all_bots default_bot_name =
    do prog_name <- getProgName
       putStrLn $ "usage: " ++ prog_name ++ " [options] [bot name]"
       putStrLn $ "  available bots: " ++ unwords (map fst all_bots)
       putStrLn $ "  default bot: " ++ default_bot_name
       putStrLn $ "  options:"
       putStrLn $ "    -h, --help: display this help message"
       putStrLn $ "    -v, --verbose: enable debug messages"
       putStrLn $ "    -s <seed>, --seed <seed>: seed random number generator with string <seed>"
       exitWith (ExitFailure 1)
