module CopAI where

import Control.Concurrent.MVar
import Monad

import AI
import State
import CopPlugins
import ForceUtil
import Dbg

data CopAI state_t = CopAI {
    cai_name :: String,
    cai_init_type :: PlayerType,
    cai_init :: CopInitAI state_t,
    cai_inform :: CopStepAI state_t WorldUpdate [Inform],
    cai_plan :: CopStepAI state_t [(PlayerName, [Maybe Inform])] [Plan],
    cai_vote :: CopStepAI state_t [(PlayerName, [Maybe Plan])] Vote,
    cai_move :: CopStepAI state_t Winner CopMove
}

type CopInitAI state_t =
    InitAI CopPlugins state_t
type CopStepAI state_t update_t result_t =
    StepAI CopPlugins state_t update_t result_t

seq_informs :: SeqFn [Inform] b
seq_informs = seq_list seq_show

seq_plans :: SeqFn [Plan] b
seq_plans = seq_list seq_show

seq_vote :: SeqFn Vote b
seq_vote = seq_show

seq_move :: SeqFn CopMove b
seq_move = seq_show

set_informs ::
    Dbg -> MVar (Maybe (plugins_t, state_t, [Inform])) ->
    plugins_t -> state_t -> ([Inform], String) -> IO ()
set_informs dbg result_var plugins state (informs, str) =
    do seq_informs informs $ set_result result_var plugins state informs
       unless (null str) $ dbgPutStrLnHead dbg $ "inform: " ++ str

set_plans ::
    Dbg -> MVar (Maybe (plugins_t, state_t, [Plan])) ->
    plugins_t -> state_t -> ([Plan], String) -> IO ()
set_plans dbg result_var plugins state (plans, str) =
    do seq_plans plans $ set_result result_var plugins state plans
       unless (null str) $ dbgPutStrLnHead dbg $ "plan: " ++ str

set_vote ::
    Dbg -> MVar (Maybe (plugins_t, state_t, Vote)) ->
    plugins_t -> state_t -> (Vote, String) -> IO ()
set_vote dbg result_var plugins state (vote, str) =
    do seq_vote vote $ set_result result_var plugins state vote
       unless (null str) $ dbgPutStrLnHead dbg $ "vote: " ++ str

set_move ::
    Dbg -> MVar (Maybe (plugins_t, state_t, CopMove)) ->
    plugins_t -> state_t -> (CopMove, String) -> IO ()
set_move dbg result_var plugins state (move, str) =
    do seq_move move $ set_result result_var plugins state move
       unless (null str) $ dbgPutStrLnHead dbg $ "move: " ++ str

