module CopAlarmed where

import State
import CopAI
import AI
import Dbg
import VoteUtil
import CopPlugins
import CopPluginCache
import CopMoveUtil
import PlanUtil
import CopPluginLog
import InformUtil

-- cop-alarmed moves to wherever the robber was last seen
-- never shares information, never submits plans,
-- and votes for himself followed by a random permutation of cop names

cop_alarmed =
    CopAI { cai_name = "Alarmed",
            cai_init_type = PT_cop_foot,
            cai_init = cop_init,
            cai_inform = cop_inform,
            cai_plan = cop_plan,
            cai_vote = cop_vote,
            cai_move = cop_move }

data AIState = AIState {
    ais_last_robber_loc :: Loc
} deriving Show

cop_init :: CopInitAI AIState
cop_init dbg plugins ws state_var =
    set_state state_var plugins
    AIState {
        ais_last_robber_loc = ws_robber_start ws
    }

cop_inform :: CopStepAI AIState WorldUpdate [Inform]
cop_inform dbg plugins state wu result_var =
    do let s' = case wu_robber_state wu of
                  Just PlayerState { ps_loc = l, ps_type = PT_robber } ->
                      state { ais_last_robber_loc = l }
                  _ -> state
       let empty_informs_str = get_empty_informs plugins
       set_informs dbg result_var plugins s' empty_informs_str

cop_plan :: CopStepAI AIState [(PlayerName, [Maybe Inform])] [Plan]
cop_plan dbg plugins state informs result_var =
    do let empty_plans_str = get_empty_plans plugins
       set_plans dbg result_var plugins state empty_plans_str

       let identity_plans_str = get_identity_plans plugins
       set_plans dbg result_var plugins state identity_plans_str

       let ws = cpi_log_skeleton (cpi_log plugins)
           wu = cpi_log_update (cpi_log plugins)
           cops = ws_cops ws
           cop_states = wu_cop_states wu
           epoch = wu_epoch wu
           robber_loc = ais_last_robber_loc state
           next_step_or_stay ps =
              case cpi_cache_plot_course (cpi_cache plugins) ps robber_loc of
              ([],_) -> error "cop_alarmed_plan: no course"
              ((here:next_step:_),_) -> next_step
              ([here],_) -> here
           moves = map next_step_or_stay cop_states
           plans = states_to_plans (wu_epoch wu) cops moves
       set_plans dbg result_var plugins state
           (plans, "toward last robber location")

cop_vote :: CopStepAI AIState [(PlayerName, [Maybe Plan])] Vote
cop_vote dbg plugins state plans result_var =
    do vote_str <- get_self_then_random_vote plugins
       set_vote dbg result_var plugins state vote_str

cop_move :: CopStepAI AIState Winner CopMove
cop_move dbg plugins state winner result_var =
    do let identity_move_str = get_identity_cop_move plugins
       set_move dbg result_var plugins state identity_move_str

       let planned_move_str = get_planned_cop_move plugins
       set_move dbg result_var plugins state planned_move_str
