module CopCritic where

import qualified Data.Set as S
import Data.Array
import Monad
import qualified Data.Map as M
import List
import Maybe

import State
import CopAI
import AI
import CopPlugins
import CopPluginLog
import CopPluginTracking
import Dbg
import VoteUtil
import CopMoveUtil
import LegalMoves
import PlanUtil
import Tracking
import ListUtil
import RandomUtil
import CopPluginCache
import InformUtil

cop_critic =
    CopAI { cai_name = "Critic",
            cai_init_type = PT_cop_foot,
            cai_init = cop_init,
            cai_inform = cop_inform,
            cai_plan = cop_plan,
            cai_vote = cop_vote,
            cai_move = cop_move }

data AIState = AIState {
    ais_dummy :: ()
}

cop_init :: CopInitAI AIState
cop_init dbg plugins ws state_var =
    set_state state_var plugins
    AIState {
        ais_dummy = ()
    }

cop_inform :: CopStepAI AIState WorldUpdate [Inform]
cop_inform dbg plugins state wu result_var =
    do let empty_informs_str = get_empty_informs plugins
       set_informs dbg result_var plugins state empty_informs_str

       let tracking_informs_str = get_tracking_informs plugins
       set_informs dbg result_var plugins state tracking_informs_str

cop_plan :: CopStepAI AIState [(PlayerName, [Maybe Inform])] [Plan]
cop_plan dbg plugins state informs result_var =
    do let empty_plans_str = get_empty_plans plugins
       set_plans dbg result_var plugins state empty_plans_str

       let identity_plans_str = get_identity_plans plugins
       set_plans dbg result_var plugins state identity_plans_str

       random_plans_str <- get_random_non_identity_plans plugins
       set_plans dbg result_var plugins state random_plans_str

       let ws = cpi_log_skeleton (cpi_log plugins)
           wu = cpi_log_update (cpi_log plugins)
           cops = ws_cops ws
           cop_states = wu_cop_states wu
           cop_idxs = [0..length cops - 1]

           evaluate_moves = make_evaluate_moves_fn plugins

       evaluated_plans_str <- get_evaluated_plans plugins evaluate_moves
       mapM_ (set_plans dbg result_var plugins state) evaluated_plans_str

       {-
       let states_to_locs states =
               S.fromList (map state_to_loc (S.toList states))

       let evidence = cpi_tracking_get_best_evidence (cpi_tracking plugins)
           evidence_state_sets = map snd evidence
           evidence_loc_sets = map states_to_locs evidence_state_sets

           robber_state_set =
               cpi_tracking_get_best_states (cpi_tracking plugins)
           robber_loc_set = states_to_locs robber_state_set

           interesting_loc_sets = evidence_loc_sets ++ [robber_loc_set]

           interesting_rev = reverse interesting_loc_sets
           interesting_descending =
               descending_by (compare_by S.size) interesting_rev
           interesting_ascending = reverse interesting_descending
           -- Here we add more copies of the robber locations because
           -- otherwise if we know exactly where the robber is only
           -- one cop moves and the rest just stand there.
           interesting = interesting_ascending ++
                         replicate (length cops) robber_loc_set

           plot_course_cached = cpi_cache_plot_course (cpi_cache plugins)

           -- Randomize me
           possible_assignments idxs s =
               [((idx, dest), course_dist)
                |
                idx <- idxs,
                let cop_state = cop_states !! idx,
                dest <- S.toList s,
                let course_dist = plot_course_cached cop_state dest]
       -}

       {-
       dbgPutStrLn dbg $
           show $ possible_assignments cop_idxs (head interesting)
       -}

       {-       
       let

           assign_cops [] _ = []
           assign_cops _ [] = []
           assign_cops idxs (int:ints)
               | S.null int = assign_cops idxs ints
               | otherwise =
                   let poss = possible_assignments idxs int
                       sorted = sortBy (compare_by (snd . snd)) poss
                       (((idx, dest), ((_:move:_), dist)):_) = sorted
                       new_idxs = delete idx idxs
                       new_ints = (S.delete dest int:ints)
                   in  (idx, move):assign_cops new_idxs new_ints

           assignments = assign_cops cop_idxs interesting

           determine_move idx =
               case lookup idx assignments of
                 Just move -> move
                 Nothing -> cop_states !! idx
           moves = map determine_move cop_idxs
           plans = states_to_plans (wu_epoch wu) cops moves
       set_plans dbg result_var plugins state (plans, "sleuth plan")
       -}

       {-
       let ws = cpi_log_skeleton (cpi_log plugins)
           wu = cpi_log_update (cpi_log plugins)
           cops = ws_cops ws
           cop_states = wu_cop_states wu
           all_moves = cartesian_product $ map (legal_moves ws) cop_states
       dbgPutStrLn dbg $ "Num moves = " ++ show (length all_moves)
       all_moves_randomized <- random_list_perm all_moves
       dbgPutStrLn dbg $ "Num randomized moves = " ++
                         show (length all_moves_randomized)
       let moves_with_scores = [(m, score) |
                                m <- all_moves_randomized,
                                let score = evaluate_moves plugins m]
           ascending_moves_with_scores =
               ascending_by (compare_by snd) moves_with_scores
           ascending_moves = map fst ascending_moves_with_scores
           ascending_plans =
               map (states_to_plans (wu_epoch wu) cops) ascending_moves

           set_eval_plans plans =
               do set_plans result_var plugins state plans
                  dbgPutStrLnHead dbg $ "plan: evaluated plan"

       --dbgPutStr dbg $ unlines $ map show ascending_moves_with_scores

       mapM set_eval_plans ascending_plans
       -}

       --dbgPutStrLn dbg "Foo"

       --evaluate_moves :: CopPlugins -> [PlayerState] -> Double

       {-
       let my_possible = cpi_tracking_get_my_states (cpi_tracking plugins)
       dbgPutStrLn dbg $
           "Mine: " ++ show (S.size my_possible)
       -}

       let best_possible = cpi_tracking_get_best_states (cpi_tracking plugins)
       dbgPutStrLn dbg $
           "Best: " ++ show (S.size best_possible)

       {-
       let non_liars = cpi_tracking_non_liars (cpi_tracking plugins)
       dbgPutStrLn dbg $ "Non-liars: " ++ show (map fromEnum non_liars)
       -}

       let show_evidence_item (epoch, states) =
               show (epoch, S.size states)

       {-
       let my_evidence = cpi_tracking_get_my_evidence
                         (cpi_tracking plugins)
       dbgPutStrLn dbg $ "Ev: mine: " ++
                         show (map show_evidence_item my_evidence)
       -}

       let best_evidence = cpi_tracking_get_best_evidence
                           (cpi_tracking plugins)
       dbgPutStrLn dbg $ "Ev: best: " ++
                         show (map show_evidence_item best_evidence)

       {-
       let my_tracking = cpi_tracking_get_my_tracking
                         (cpi_tracking plugins)
           my_tracking_list = elems my_tracking
           my_size_list = map S.size my_tracking_list
       dbgPutStrLn dbg $ "Mine: " ++ show my_size_list
       -}

       let best_tracking = cpi_tracking_get_best_tracking
                           (cpi_tracking plugins)
           best_tracking_list = elems best_tracking
           best_size_list = map S.size best_tracking_list
       dbgPutStrLn dbg $ "Best: " ++ show best_size_list

       dbgPutStrLn dbg $ "Evidence senders: " ++
                   show (cpi_tracking_evidence_senders (cpi_tracking plugins))

       {-
       let tracking_all = cpi_tracking_all (cpi_tracking plugins)
           tracking_all_list = M.assocs tracking_all
           show_map_entry (key, value) =
               show (map fromEnum key) ++ show (map S.size (elems value))
       dbgPutStr dbg $ unlines $ map show_map_entry tracking_all_list
       -}

make_evaluate_moves_fn :: CopPlugins -> [PlayerState] -> Int
make_evaluate_moves_fn plugins =
    let --ws = cpi_log_skeleton (cpi_log plugins)
        wu = cpi_log_update (cpi_log plugins)
        --cops = ws_cops ws
        cur_cop_states = wu_cop_states wu

        robber_state_set =
            cpi_tracking_get_best_states (cpi_tracking plugins)
        robber_locs = nub $ map state_to_loc (S.toList robber_state_set)

        states_to_locs states =
            S.fromList (map state_to_loc (S.toList states))

        evidence = cpi_tracking_get_best_evidence (cpi_tracking plugins)
        evidence_state_sets = map snd evidence
        evidence_loc_sets = map states_to_locs evidence_state_sets

        --robber_state_set =
        --    cpi_tracking_get_best_states (cpi_tracking plugins)
        --robber_loc_set = states_to_locs robber_state_set

        ev_rev = reverse evidence_loc_sets
        ev_descending =
            descending_by (compare_by S.size) ev_rev
        ev_ascending = reverse ev_descending

        ev = [one_ev |
              one_ev <- ev_ascending,
              S.size one_ev <= 30,
              S.size one_ev > 0] -- just to be safe

        ev_locs = S.unions ev
        get_ev_loc_score loc = sum [100 `div` (S.size one_ev)
                                    |
                                    one_ev <- ev,
                                    S.member loc one_ev]
        ev_loc_scores = [(loc, score)
                         |
                         loc <- S.toList ev_locs,
                         let score = get_ev_loc_score loc]


        evaluate_moves cop_states =
            let cop_locs = map state_to_loc cop_states
            in  eval_can_reach_all_robber_locs cop_locs +
                --eval_evidence cop_locs +
                0

        eval_can_reach_all_robber_locs cop_locs =
            let is_a_cop_loc loc = elem loc cop_locs
            in  if all is_a_cop_loc robber_locs
                then 1000000
                else 0

        plot_course_cached = cpi_cache_plot_course (cpi_cache plugins)
        dist_to_loc state loc =
            let (course, dist) = plot_course_cached state loc
            in  dist

        {-
        eval_cops_near_evidence cop_states =
            sum [score |
                 (loc, score) <- ev_loc_scores,
                 elem loc cop_locs]
        -}

    in  evaluate_moves

{-
evaluate_moves :: CopPlugins -> [PlayerState] -> Double
evaluate_moves plugins moves =
    let evidence = cpi_tracking_get_best_evidence (cpi_tracking plugins)
        evidence_state_sets = map snd evidence
        robber_state_set = cpi_tracking_get_best_states (cpi_tracking plugins)
        interesting_state_sets = evidence_state_sets ++ [robber_state_set]
        states_to_locs states =
            S.fromList (nub $ map state_to_loc (S.toList states))
        interesting_loc_sets = map states_to_locs interesting_state_sets

        unique_move_locs = nub $ map state_to_loc moves

        score :: Double
        score = sum [1 / fromInteger (toInteger (S.size set))
                     |
                     move_loc <- unique_move_locs,
                     set <- interesting_loc_sets,
                     move_loc `S.member` set]
    in  score
-}

cop_vote :: CopStepAI AIState [(PlayerName, [Maybe Plan])] Vote
cop_vote dbg plugins state plans result_var =
    do random_vote_str <- get_self_then_random_vote plugins
       set_vote dbg result_var plugins state random_vote_str

       let evaluate_moves = make_evaluate_moves_fn plugins

       evaluated_vote_str@(evaluated_vote, _)
           <- get_evaluated_vote plugins evaluate_moves
       set_vote dbg result_var plugins state evaluated_vote_str

       dbgPutStrLn dbg $ "Vote = " ++ show evaluated_vote

cop_move :: CopStepAI AIState Winner CopMove
cop_move dbg plugins state winner result_var =
    do let identity_move_str = get_identity_cop_move plugins
       set_move dbg result_var plugins state identity_move_str

       {-
       random_move_str <- get_random_non_identity_move plugins
       set_move dbg result_var plugins state random_move_str
       -}

       let planned_move_str = get_planned_cop_move plugins
       set_move dbg result_var plugins state planned_move_str

       let maybe_voted_move_str = get_voted_cop_move plugins
       case maybe_voted_move_str of
         Nothing -> return ()
         Just voted_move_str ->
             set_move dbg result_var plugins state voted_move_str
