module CopFollower where

import Data.Maybe
import Data.List
import qualified Data.Set as S

import State
import CopAI
import AI
import Dbg
import RandomUtil
import PlotCourse
import CopPlugins
import CopPluginLog
import VoteUtil
import PlanUtil
import CopPluginTracking
import CopMoveUtil
import CopPluginCache
import InformUtil

cop_follower =
    CopAI { cai_name = "Follower",
            cai_init_type = PT_cop_foot,
            cai_init = cop_init,
            cai_inform = cop_inform,
            cai_plan = cop_plan,
            cai_vote = cop_vote,
            cai_move = cop_move }

data AIState = AIState {
    ais_dummy :: ()
}

cop_init :: CopInitAI AIState
cop_init dbg plugins ws state_var =
    set_state state_var plugins
    AIState {
        ais_dummy = ()
    }

cop_inform :: CopStepAI AIState WorldUpdate [Inform]
cop_inform dbg plugins state wu result_var =
    do let empty_informs_str = get_empty_informs plugins
       set_informs dbg result_var plugins state empty_informs_str

       let tracking_informs_str = get_tracking_informs plugins
       set_informs dbg result_var plugins state tracking_informs_str

cop_plan :: CopStepAI AIState [(PlayerName, [Maybe Inform])] [Plan]
cop_plan dbg plugins state informs result_var =
    do let identity_plans_str = get_identity_plans plugins
       set_plans dbg result_var plugins state identity_plans_str

       random_plans_str <- get_random_plans plugins
       set_plans dbg result_var plugins state random_plans_str

       let ws = cpi_log_skeleton (cpi_log plugins)
           wu = cpi_log_update (cpi_log plugins)
           cops = ws_cops ws
           cop_states = wu_cop_states wu

       let poss_robber_states_set =
               cpi_tracking_get_best_states (cpi_tracking plugins)
           poss_robber_states = S.toList poss_robber_states_set
           poss_robber_locs :: [Loc]
           poss_robber_locs = nub $ map state_to_loc poss_robber_states

       let next_step_or_stay ps =
              do rand_loc <- random_list_elt poss_robber_locs
                 case cpi_cache_plot_course (cpi_cache plugins) ps rand_loc of
                   ([],_) -> error "cop_alarmed_plan: no course"
                   ((here:next_step:_),_) -> return next_step
                   ([here],_) -> return here
       next_ps_list <- mapM next_step_or_stay cop_states
       let real_plans =
               states_to_plans (wu_epoch wu) cops next_ps_list
       set_plans dbg result_var plugins state
           (real_plans, "random step toward a possible robber location")

cop_vote :: CopStepAI AIState [(PlayerName, [Maybe Plan])] Vote
cop_vote dbg plugins state plans result_var =
    do vote_str <- get_self_then_random_vote plugins
       set_vote dbg result_var plugins state vote_str

cop_move :: CopStepAI AIState Winner CopMove
cop_move dbg plugins state winner result_var =
    do let identity_move_str = get_identity_cop_move plugins
       set_move dbg result_var plugins state identity_move_str

       let planned_move_str = get_planned_cop_move plugins
       set_move dbg result_var plugins state planned_move_str
