module CopLiar where

import State
import CopAI
import AI
import Dbg
import VoteUtil
import CopMoveUtil
import CopPlugins
import CopPluginLog
import InformUtil
import PlanUtil

cop_liar =
    CopAI { cai_name = "Liar",
            cai_init_type = PT_cop_foot,
            cai_init = cop_init,
            cai_inform = cop_inform,
            cai_plan = cop_plan,
            cai_vote = cop_vote,
            cai_move = cop_move }

data AIState = AIState {
    ais_dummy :: ()
}

cop_init :: CopInitAI AIState
cop_init dbg plugins ws state_var =
    set_state state_var plugins
    AIState {
        ais_dummy = ()
    }

cop_inform :: CopStepAI AIState WorldUpdate [Inform]
cop_inform dbg plugins state wu result_var =
    do let empty_informs_str = get_empty_informs plugins
       set_informs dbg result_var plugins state empty_informs_str

       let ws = cpi_log_skeleton (cpi_log plugins)
           wu = cpi_log_update (cpi_log plugins)
           epoch = wu_epoch wu
           informs =
               [Inform { if_who = ws_robber ws,
                         if_loc = ws_robber_start ws,
                         if_ptype = PT_robber,
                         if_epoch = epoch,
                         if_certainty = 100 } ]

       set_informs dbg result_var plugins state
           (informs, "lies, damn lies, and statistics")

cop_plan :: CopStepAI AIState [(PlayerName, [Maybe Inform])] [Plan]
cop_plan dbg plugins state informs result_var =
    do let empty_plans_str = get_empty_plans plugins
       set_plans dbg result_var plugins state empty_plans_str

cop_vote :: CopStepAI AIState [(PlayerName, [Maybe Plan])] Vote
cop_vote dbg plugins state plans result_var =
    do vote_str <- get_random_vote plugins
       set_vote dbg result_var plugins state vote_str

cop_move :: CopStepAI AIState Winner CopMove
cop_move dbg plugins state winner result_var =
    do let identity_move_str = get_identity_cop_move plugins
       set_move dbg result_var plugins state identity_move_str

       let maybe_voted_move_str = get_voted_cop_move plugins
       case maybe_voted_move_str of
         Nothing -> return ()
         Just voted_move_str ->
             set_move dbg result_var plugins state voted_move_str
