module CopPluginsImpl where

import State
import CopPlugins
import CopPluginLogImpl
import CopPluginCacheImpl
import CopPluginTrackingImpl

cop_plugins_init :: WorldSkeleton -> CopPlugins
cop_plugins_init ws =
    plugins where plugins = CopPlugins {
        cpi_log = cpi_log_init plugins ws,
        cpi_cache = cpi_cache_init plugins ws,
        cpi_tracking = cpi_tracking_init plugins ws,
        cpi_dummy = ()
    }

cop_plugins_pre_inform :: CopPlugins -> WorldUpdate -> CopPlugins
cop_plugins_pre_inform old wu =
    new where new = old {
        cpi_log = cpi_log_pre_inform old new (cpi_log old) wu,
        cpi_tracking = cpi_tracking_pre_inform old new (cpi_tracking old) wu,
        cpi_dummy = ()
    }

cop_plugins_post_inform :: CopPlugins -> [Inform] -> CopPlugins
cop_plugins_post_inform old informs =
    new where new = old {
        cpi_log = cpi_log_post_inform old new (cpi_log old) informs,
        cpi_dummy = ()
    }

cop_plugins_pre_plan ::
    CopPlugins -> [(PlayerName, [Maybe Inform])] -> CopPlugins
cop_plugins_pre_plan old informs =
    new where new = old {
        cpi_log = cpi_log_pre_plan old new (cpi_log old) informs,
        cpi_tracking =
            cpi_tracking_pre_plan old new (cpi_tracking old) informs,
        cpi_dummy = ()
    }

cop_plugins_post_plan :: CopPlugins -> [Plan] -> CopPlugins
cop_plugins_post_plan old plans =
    new where new = old {
        cpi_log = cpi_log_post_plan old new (cpi_log old) plans,
        cpi_dummy = ()
    }

cop_plugins_pre_vote ::
    CopPlugins -> [(PlayerName, [Maybe Plan])] -> CopPlugins
cop_plugins_pre_vote old plans =
    new where new = old {
        cpi_log = cpi_log_pre_vote old new (cpi_log old) plans,
        cpi_dummy = ()
    }

cop_plugins_post_vote :: CopPlugins -> [PlayerName] -> CopPlugins
cop_plugins_post_vote old vote =
    new where new = old {
        cpi_log = cpi_log_post_vote old new (cpi_log old) vote,
        cpi_dummy = ()
    }

cop_plugins_pre_move :: CopPlugins -> Winner -> CopPlugins
cop_plugins_pre_move old winner =
    new where new = old {
        cpi_log = cpi_log_pre_move old new (cpi_log old) winner,
        cpi_dummy = ()
    }

cop_plugins_post_move :: CopPlugins -> CopMove -> CopPlugins
cop_plugins_post_move old move =
    new where new = old {
        cpi_log = cpi_log_post_move old new (cpi_log old) move,
        cpi_dummy = ()
    }
