module PlanUtil where

import Maybe

import CopPlugins
import State
import CopPluginLog
import LegalMoves
import CopPluginCache
import ListUtil
import RandomUtil

states_to_plans :: Epoch -> [PlayerName] -> [PlayerState] -> [Plan]
states_to_plans epoch cops cop_states =
    map make_plan (zip cops cop_states)
    where
    make_plan (cop, state) =
         Plan { pl_epoch = epoch + 1,
                pl_move = PlayerMove { pm_name = cop, pm_state = state }}

get_empty_plans :: CopPlugins -> ([Plan], String)
get_empty_plans plugins =
    ([], "nothing")

get_identity_plans :: CopPlugins -> ([Plan], String)
get_identity_plans plugins =
    let ws = cpi_log_skeleton (cpi_log plugins)
        wu = cpi_log_update (cpi_log plugins)
        cops = ws_cops ws
        cop_states = wu_cop_states wu
        plans = states_to_plans (wu_epoch wu) cops cop_states
    in  (plans, "identity")

get_random_plans :: CopPlugins -> IO ([Plan], String)
get_random_plans plugins =
    do let ws = cpi_log_skeleton (cpi_log plugins)
           wu = cpi_log_update (cpi_log plugins)
           cops = ws_cops ws
           cop_states = wu_cop_states wu
       random_states <- mapM (random_legal_move ws) cop_states
       let plans = states_to_plans (wu_epoch wu) cops random_states
       return (plans, "random")

get_random_non_identity_plans :: CopPlugins -> IO ([Plan], String)
get_random_non_identity_plans plugins =
    do let ws = cpi_log_skeleton (cpi_log plugins)
           wu = cpi_log_update (cpi_log plugins)
           cops = ws_cops ws
           cop_states = wu_cop_states wu
       random_states <- mapM (random_non_identity_legal_move ws) cop_states
       let plans = states_to_plans (wu_epoch wu) cops random_states
       return (plans, "random non-identity")

get_evaluated_plans :: (Ord score_t, Show score_t) =>
                       CopPlugins -> ([PlayerState] -> score_t) ->
                       IO [([Plan], String)]
get_evaluated_plans plugins eval_fn =
    do let ws = cpi_log_skeleton (cpi_log plugins)
           wu = cpi_log_update (cpi_log plugins)
           cops = ws_cops ws
           cop_states = wu_cop_states wu
           legal_moves_cached =
               cpi_cache_legal_moves (cpi_cache plugins)
           next_states = map legal_moves_cached cop_states
       randomized_next_states <- mapM random_list_perm next_states
       let all_moves = cartesian_product randomized_next_states
           moves_with_scores = [(move, score) |
                                move <- all_moves,
                                let score = eval_fn move]
           ascending_moves_with_scores =
               ascending_by (compare_by snd) moves_with_scores

           {-
           ascending_moves = map fst ascending_moves_with_scores
           ascending_plans =
               map (states_to_plans (wu_epoch wu) cops) ascending_moves
           -}

           plans_msg = [(plan, msg) |
                        (move, score) <- ascending_moves_with_scores,
                        let plan = states_to_plans (wu_epoch wu) cops move,
                        let msg = "evaluated plan, score = " ++ show score]

           {-
           make_plan_msg (plan, i) =
               (plan, "evaluated plan " ++ show i)

           plans_msg = map make_plan_msg (zip ascending_plans [0..])
           -}

       return plans_msg
