module ProtocolCop where

import Control.Concurrent.MVar

import Protocol
import State
import Parser
import CopAI
import CopPlugins
import CopPluginsImpl
import Dbg

run_cop :: CopAI state_t -> Bool -> IO ()
run_cop ai verbose =
    do state_var <- newEmptyMVar
       inform_var <- newEmptyMVar
       plan_var <- newEmptyMVar
       vote_var <- newEmptyMVar
       move_var <- newEmptyMVar
       send_reg (cai_name ai) (cai_init_type ai)
       (skel, name_to_loc, loc_to_name, cops, robber) <- read_skeleton
       let dbg = Dbg { dbg_verbose = verbose,
                       dbg_head = "Cop " ++ ws_my_name skel ++ ": " }
           plugins0 = cop_plugins_init skel
       (plugins0', state) <- run_step ((cai_init ai) dbg plugins0 skel)
                             state_var dbg "init"
       run_cop_loop name_to_loc loc_to_name cops robber dbg
                    ai state plugins0' inform_var plan_var vote_var move_var

run_cop_loop :: NameToLoc ->
                LocToName ->
                [PlayerName] ->
                PlayerName ->
                Dbg ->
                CopAI state_t ->
                state_t ->
                CopPlugins ->
                MVar (Maybe (CopPlugins, state_t, [Inform])) ->
                MVar (Maybe (CopPlugins, state_t, [Plan])) ->
                MVar (Maybe (CopPlugins, state_t, Vote)) ->
                MVar (Maybe (CopPlugins, state_t, CopMove)) ->
                IO ()
run_cop_loop name_to_loc loc_to_name cops robber dbg
             ai state0 plugins0 inform_var plan_var vote_var move_var =
    do update <- read_update name_to_loc cops robber
       dbgPutStrLn dbg $ "=== Epoch: " ++ show (wu_epoch update) ++ " ==="
       let plugins1 = cop_plugins_pre_inform plugins0 update
       (plugins1', state1, my_informs) <-
           run_step ((cai_inform ai) dbg plugins1 state0 update)
                    inform_var dbg "inform"
       let plugins1'' = cop_plugins_post_inform plugins1' my_informs
       send_informs loc_to_name my_informs
       informs <- read_informs name_to_loc cops
       let plugins2 = cop_plugins_pre_plan plugins1'' informs
       (plugins2', state2, my_plans) <-
           run_step ((cai_plan ai) dbg plugins2 state1 informs)
                    plan_var dbg "plan"
       let plugins2'' = cop_plugins_post_plan plugins2' my_plans
       send_plans loc_to_name my_plans
       plans <- read_plans name_to_loc cops
       let plugins3 = cop_plugins_pre_vote plugins2'' plans
       (plugins3', state3, my_vote) <-
           run_step ((cai_vote ai) dbg plugins3 state2 plans)
                    vote_var dbg "vote"
       let plugins3'' = cop_plugins_post_vote plugins3' my_vote
       send_vote my_vote
       winner <- read_winner
       let plugins4 = cop_plugins_pre_move plugins3'' winner
       (plugins4', state4, my_move) <-
           run_step ((cai_move ai) dbg plugins4 state3 winner)
                    move_var dbg "move"
       let plugins4'' = cop_plugins_post_move plugins4' my_move
       send_cop_move loc_to_name my_move
       run_cop_loop name_to_loc loc_to_name cops robber dbg
                    ai state4 plugins4'' inform_var plan_var vote_var move_var
