module ProtocolRobber where

import Control.Concurrent.MVar

import Protocol
import State
import Parser
import RobberAI
import RobberPlugins
import RobberPluginsImpl
import Dbg

run_robber :: RobberAI state_t -> Bool -> IO ()
run_robber ai verbose =
    do state_var <- newEmptyMVar
       inform_var <- newEmptyMVar
       plan_var <- newEmptyMVar
       vote_var <- newEmptyMVar
       bribe_var <- newEmptyMVar
       move_var <- newEmptyMVar
       send_reg (rai_name ai) PT_robber
       (skel, name_to_loc, loc_to_name, cops, robber) <- read_skeleton
       let dbg = Dbg { dbg_verbose = verbose,
                       dbg_head = "Robber " ++ ws_my_name skel ++ ": " }
           plugins0 = robber_plugins_init skel
       (plugins0', state) <- run_step ((rai_init ai) dbg plugins0 skel)
                             state_var dbg "init"
       run_robber_loop name_to_loc loc_to_name cops robber dbg
                       ai state plugins0' inform_var plan_var vote_var
                       bribe_var move_var

run_robber_loop :: NameToLoc ->
                LocToName ->
                [PlayerName] ->
                PlayerName ->
                Dbg ->
                RobberAI state_t ->
                state_t ->
                RobberPlugins ->
                MVar (Maybe (RobberPlugins, state_t, [Inform])) ->
                MVar (Maybe (RobberPlugins, state_t, [Plan])) ->
                MVar (Maybe (RobberPlugins, state_t, Vote)) ->
                MVar (Maybe (RobberPlugins, state_t, Bool)) ->
                MVar (Maybe (RobberPlugins, state_t, RobberMove)) ->
                IO ()
run_robber_loop name_to_loc loc_to_name scoundrels robber dbg
           ai state0 plugins0 inform_var plan_var vote_var bribe_var move_var =
    do update <- read_update name_to_loc scoundrels robber
       dbgPutStrLn dbg $ "=== Epoch: " ++ show (wu_epoch update) ++ " ==="
       let plugins1 = robber_plugins_pre_inform plugins0 update
       (plugins1', state1, my_informs) <-
           run_step ((rai_inform ai) dbg plugins1 state0 update)
                    inform_var dbg "inform"
       let plugins1'' = robber_plugins_post_inform plugins1' my_informs
       send_informs loc_to_name my_informs
       informs <- read_informs name_to_loc scoundrels
       let plugins2 = robber_plugins_pre_plan plugins1'' informs
       (plugins2', state2, my_plans) <-
           run_step ((rai_plan ai) dbg plugins2 state1 informs)
                    plan_var dbg "plan"
       let plugins2'' = robber_plugins_post_plan plugins2' my_plans
       send_plans loc_to_name my_plans
       plans <- read_plans name_to_loc scoundrels
       let plugins3 = robber_plugins_pre_vote plugins2'' plans
       (plugins3', state3, my_vote) <-
           run_step ((rai_vote ai) dbg plugins3 state2 plans)
                    vote_var dbg "vote"
       let plugins3'' = robber_plugins_post_vote plugins3' my_vote
       send_vote my_vote
       winner <- read_winner
       let plugins4 = robber_plugins_pre_bribe plugins3'' winner
       (plugins4', state4, my_bribe) <-
           run_step ((rai_bribe ai) dbg plugins4 state3 winner)
                    bribe_var dbg "bribe"
       let plugins4'' = robber_plugins_post_bribe plugins4' my_bribe
       send_bribe my_bribe
       offers <- read_offers
       let plugins5 = robber_plugins_pre_move plugins3'' offers
       (plugins5', state5, my_move) <-
           run_step ((rai_move ai) dbg plugins5 state4 offers)
                    move_var dbg "move"
       let plugins5'' = robber_plugins_post_move plugins5' my_move
       send_robber_move loc_to_name my_move
       run_robber_loop name_to_loc loc_to_name scoundrels robber dbg
           ai state5 plugins5'' inform_var plan_var vote_var bribe_var move_var

