module RobberAI where

import Control.Concurrent.MVar
import Monad

import AI
import State
import RobberPlugins
import ForceUtil
import Dbg

data RobberAI state_t = RobberAI {
    rai_name :: String,
    rai_init :: RobberInitAI state_t,
    rai_inform :: RobberStepAI state_t WorldUpdate [Inform],
    rai_plan :: RobberStepAI state_t [(PlayerName, [Maybe Inform])] [Plan],
    rai_vote :: RobberStepAI state_t [(PlayerName, [Maybe Plan])] Vote,
    rai_bribe :: RobberStepAI state_t Winner Bool,
    rai_move :: RobberStepAI state_t [PlayerName] RobberMove
}

type RobberInitAI state_t =
    InitAI RobberPlugins state_t
type RobberStepAI state_t update_t result_t =
    StepAI RobberPlugins state_t update_t result_t

seq_move :: SeqFn RobberMove b
seq_move = seq_show

set_move ::
    Dbg -> MVar (Maybe (plugins_t, state_t, RobberMove)) ->
    plugins_t -> state_t -> (RobberMove, String) -> IO ()
set_move dbg result_var plugins state (move, str) =
    do seq_move move $ set_result result_var plugins state move
       unless (null str) $ dbgPutStrLnHead dbg $ "move: " ++ str

seq_informs :: SeqFn [Inform] b
seq_informs = seq_list seq_show

seq_plans :: SeqFn [Plan] b
seq_plans = seq_list seq_show

seq_vote :: SeqFn Vote b
seq_vote = seq_show

seq_bribe :: SeqFn Bool b
seq_bribe = seq_show

set_informs ::
    Dbg -> MVar (Maybe (plugins_t, state_t, [Inform])) ->
    plugins_t -> state_t -> ([Inform], String) -> IO ()
set_informs dbg result_var plugins state (informs, str) =
    do seq_informs informs $ set_result result_var plugins state informs
       unless (null str) $ dbgPutStrLnHead dbg $ "inform: " ++ str

set_plans ::
    Dbg -> MVar (Maybe (plugins_t, state_t, [Plan])) ->
    plugins_t -> state_t -> ([Plan], String) -> IO ()
set_plans dbg result_var plugins state (plans, str) =
    do seq_plans plans $ set_result result_var plugins state plans
       unless (null str) $ dbgPutStrLnHead dbg $ "plan: " ++ str

set_vote ::
    Dbg -> MVar (Maybe (plugins_t, state_t, Vote)) ->
    plugins_t -> state_t -> (Vote, String) -> IO ()
set_vote dbg result_var plugins state (vote, str) =
    do seq_vote vote $ set_result result_var plugins state vote
       unless (null str) $ dbgPutStrLnHead dbg $ "vote: " ++ str

set_bribe ::
    Dbg -> MVar (Maybe (plugins_t, state_t, Bool)) ->
    plugins_t -> state_t -> (Bool, String) -> IO ()
set_bribe dbg result_var plugins state (bribe, str) =
    do seq_bribe bribe $ set_result result_var plugins state bribe
       unless (null str) $ dbgPutStrLnHead dbg $ "bribe: " ++ str
