module RobberFearless where

import List

import State
import RobberAI
import AI
import Dbg
import RobberPlugins
import RobberPluginCache
import RobberMoveUtil
import RobberPluginLog

-- robber-fearless goes for the richest bank

robber_fearless =
    RobberAI { rai_name = "Fearless",
               rai_init = robber_init,
               rai_inform = robber_inform,
               rai_plan = robber_plan,
               rai_vote = robber_vote,
               rai_bribe = robber_bribe,
               rai_move = robber_move }

data AIState = AIState {
    ais_dummy :: ()
}

robber_init :: RobberInitAI AIState
robber_init dbg plugins ws state_var =
    set_state state_var plugins
    AIState {
        ais_dummy = ()
    }

robber_inform :: RobberStepAI AIState WorldUpdate [Inform]
robber_inform dbg plugins state wu inform_var =
    set_informs dbg inform_var plugins state ([], "none")

robber_plan :: RobberStepAI state_t [(PlayerName, [Maybe Inform])] [Plan]
robber_plan dbg plugins state informs plan_var =
    set_plans dbg plan_var plugins state ([], "none")

robber_vote :: RobberStepAI state_t [(PlayerName, [Maybe Plan])] Vote
robber_vote dbg plugins state informs vote_var =
    do let ws = rpi_log_skeleton (rpi_log plugins)
           wu = rpi_log_update (rpi_log plugins)
           scoundrels = ws_my_name ws : wu_dirty_cops wu
       set_vote dbg vote_var plugins state (scoundrels, "none")

robber_bribe :: RobberStepAI state_t Winner Bool
robber_bribe dbg plugins state informs bribe_var =
    set_bribe dbg bribe_var plugins state (False, "no")

robber_move :: RobberStepAI AIState [PlayerName] RobberMove
robber_move dbg plugins state offered_cops result_var =
    do let identity_move_str = get_identity_move plugins
       set_move dbg result_var plugins state identity_move_str

       let wu = rpi_log_update (rpi_log plugins)
           Just here = wu_robber_state wu
       let plot_course_cached = rpi_cache_plot_course (rpi_cache plugins)
           ((_, _, _, bank_path):_) =
               sort [(-val, cost, loc, path)
                     | (loc,val) <- wu_bank_values wu,
                       let (path,cost) = plot_course_cached here loc]
           move = case bank_path of
                  (here : next : _) -> next
                  [here] -> here
                  [] -> error "robber_fearless_move: phantom bank"
           robber_move = state_to_robber_move plugins move
       set_move dbg result_var plugins state
           (robber_move, "toward richest bank")

       let default_move = robber_move
       search_moves <- get_search_moves plugins default_move
       mapM_ (set_move dbg result_var plugins state) search_moves
