module RobberHermit where

import Maybe

import State
import RobberAI
import AI
import Dbg
import RobberMoveUtil
import RobberPlugins
import RobberPluginLog
import PlotCourse
import RobberPluginCache
import GraphUtil

robber_hermit =
    RobberAI { rai_name = "Hermit",
               rai_init = robber_init,
               rai_inform = robber_inform,
               rai_plan = robber_plan,
               rai_vote = robber_vote,
               rai_bribe = robber_bribe,
               rai_move = robber_move }

data AIState = AIState {
    ais_dest :: Loc,
    ais_dummy :: ()
}

robber_init :: RobberInitAI AIState
robber_init dbg plugins ws state_var =
    set_state state_var plugins
    AIState {
        -- It would be nice not to hard code the hiding place.
        ais_dest = fromJust (ws_name_to_loc ws "museum-loop-#3"),
        ais_dummy = ()
    }

robber_inform :: RobberStepAI AIState WorldUpdate [Inform]
robber_inform dbg plugins state wu inform_var =
    set_informs dbg inform_var plugins state ([], "none")

robber_plan :: RobberStepAI state_t [(PlayerName, [Maybe Inform])] [Plan]
robber_plan dbg plugins state informs plan_var =
    set_plans dbg plan_var plugins state ([], "none")

robber_vote :: RobberStepAI state_t [(PlayerName, [Maybe Plan])] Vote
robber_vote dbg plugins state informs vote_var =
    do let ws = rpi_log_skeleton (rpi_log plugins)
           wu = rpi_log_update (rpi_log plugins)
           scoundrels = ws_my_name ws : wu_dirty_cops wu
       set_vote dbg vote_var plugins state (scoundrels, "none")

robber_bribe :: RobberStepAI state_t Winner Bool
robber_bribe dbg plugins state informs bribe_var =
    set_bribe dbg bribe_var plugins state (False, "no")

robber_move :: RobberStepAI AIState [PlayerName] RobberMove
robber_move dbg plugins state offered_cops result_var =
    do let identity_move_str@(identity_move, _) = get_identity_move plugins
       set_move dbg result_var plugins state identity_move_str

       let ws = rpi_log_skeleton (rpi_log plugins)
           wu = rpi_log_update (rpi_log plugins)
           Just my_ps = wu_robber_state wu

           bank_locs = ws_banks ws
           bank_states = map (loc_to_state PT_robber) bank_locs
           bank_nodes = map (ws_ps_to_node ws) bank_states
           bank_costs = zip bank_nodes (repeat 1000)
           map_no_banks = repl_edges_to (ws_map_ps ws) bank_costs

           (course, dist) =
               plot_course_with_map ws map_no_banks my_ps (ais_dest state)

           hermit_move_str@(hermit_move, _) =
               case course of
                 (_:course_move:_) ->
                     (state_to_robber_move plugins course_move,
                      "toward hermitage")
                 _ -> (identity_move, "already at hermitage")

       set_move dbg result_var plugins state hermit_move_str

       let default_move = hermit_move
       search_moves <- get_search_moves plugins default_move
       mapM_ (set_move dbg result_var plugins state) search_moves
