module RobberPluginCacheImpl where

import State
import RobberPluginCache
import RobberPlugins
import PlotCourse
import LegalMoves

rpi_cache_init :: RobberPlugins -> WorldSkeleton -> RobberPluginCache
rpi_cache_init _ ws =
    RobberPluginCache {
        rpi_cache_legal_moves =
            make_legal_moves_cache_fn ws (legal_moves ws),
        rpi_cache_legal_moves_same_state =
            make_legal_moves_cache_fn ws (legal_moves_same_state ws),
        rpi_cache_legal_anti_moves =
            make_legal_moves_cache_fn ws (legal_anti_moves ws),

        rpi_cache_plot_course = make_plot_course_cache ws (plot_course ws),

        rpi_cache_states_up_to_distance = states_up_to_distance,
        rpi_cache_states_at_distance = states_at_distance,
        rpi_cache_locs_up_to_distance = locs_up_to_distance,
        rpi_cache_locs_at_distance = locs_at_distance,

        rpi_cache_states_up_to_distance_same_state =
            states_up_to_distance_same_state,
        rpi_cache_states_at_distance_same_state =
            states_at_distance_same_state,
        rpi_cache_locs_up_to_distance_same_state =
            locs_up_to_distance_same_state,
        rpi_cache_locs_at_distance_same_state =
            locs_at_distance_same_state
    }
    where
    ((states_up_to_distance, states_at_distance),
     (locs_up_to_distance, locs_at_distance)) =
        make_by_distance_cache_fns ws (legal_moves ws)
    ((states_up_to_distance_same_state, states_at_distance_same_state),
     (locs_up_to_distance_same_state, locs_at_distance_same_state)) =
        make_by_distance_cache_fns ws (legal_moves_same_state ws)
