module VoteUtil where

import List
import Maybe

import CopPlugins
import State
import CopPluginLog
import RandomUtil
import ListUtil
import CopMoveUtil -- for get_move_from_plans

get_random_vote :: CopPlugins -> IO (Vote, String)
get_random_vote plugins =
    do let received_plans = cpi_log_received_plans (cpi_log plugins)
           cops_to_use = map fst received_plans
       vote <- random_list_perm cops_to_use
       return (vote, "random")

get_self_then_random_vote :: CopPlugins -> IO (Vote, String)
get_self_then_random_vote plugins =
    do let ws = cpi_log_skeleton (cpi_log plugins)
           moi = ws_my_name ws
           received_plans = cpi_log_received_plans (cpi_log plugins)
           cops_to_use = map fst received_plans
           cops_sans_moi = delete moi cops_to_use
       rest_vote <- random_list_perm cops_sans_moi
       let vote = (moi:rest_vote)
       return (vote, "self then random")

get_evaluated_vote :: (Ord score_t, Show score_t) =>
                       CopPlugins -> ([PlayerState] -> score_t) ->
                       IO (Vote, String)
get_evaluated_vote plugins eval_fn =
    do let received_plans = cpi_log_received_plans (cpi_log plugins)
           cops_to_use = map fst received_plans
       randomized_cops <- random_list_perm cops_to_use
       let sorted_cops = reverse $ sortBy (compare_by eval_cop) randomized_cops
       return (sorted_cops, "evaluated")
    where
    eval_cop cop_name =
        let ws = cpi_log_skeleton (cpi_log plugins)
            my_name = ws_my_name ws
            received_plans = cpi_log_received_plans (cpi_log plugins)
            maybe_plans = fromJust $ lookup cop_name received_plans
            plans = [fromJust maybe_plan
                     |
                     maybe_plan <- maybe_plans,
                     isJust maybe_plan]
            cops = ws_cops ws
            maybe_moves = map (get_move_from_plans plugins plans) cops
        in  if all isJust maybe_moves
               then let moves = map fromJust maybe_moves
                    in  (1, Just (eval_fn moves),
                         if cop_name == my_name then 1 else 0)
               else (0, Nothing, 0)
