module Main where

import Data.Maybe
import Random
import qualified Data.Set as S
import Data.Array

import LegalMoves
import Protocol
import Parser
import State
import Data.Graph.Inductive
import Minimax
import CatchSearch
import RobberPluginsImpl
import RobberPlugins
import RobberPluginCache
import Tracking

main :: IO ()
main =
    do putStrLn "Hello, world!"
       (ws, name_to_loc, loc_to_name, cops, robber) <-
           parse_from_stdin p_world_skeleton (== "wsk/")
       putStrLn $ loc_to_name 82
       putStrLn $ show $ ws_hq ws
       putStrLn $ show $ sp 1 2 (ws_map_ps ws)
       let --loc = "47-and-lake-shore" -- ws_hq ws -- ws_robber_start ws
           --playerType = PT_cop_car -- PT_robber
           --ps = PlayerState {ps_loc = loc,
           --                  ps_type = playerType}
           --newStates = legal_moves ws ps
           copState = PlayerState { ps_loc = ws_hq ws, --"54-and-kenwood",
                                    ps_type = PT_cop_foot }
           copState' = PlayerState { ps_loc = fromJust $
                                              name_to_loc "55-and-kimbark",
                                    ps_type = PT_cop_foot }
           robberState = PlayerState { ps_loc = fromJust $
                                                name_to_loc "rochdale-and-dorchester",
                                           --ws_robber_start ws,
                                           --"54-and-ridgewood",
                                       ps_type = PT_robber }
           (states_up_to_distance, states_at_distance) =
               states_by_distance ws copState
           ps_to_str ps = show (ps_type ps) ++ "@" ++ loc_to_name (ps_loc ps)

           i1 = (256*256*256) :: Int
           i2 = 1000*i1
           i3 = (fromInteger 1000000000001) :: Int
       putStrLn $ show i3
       foo <- getStdGen
       putStrLn $ show foo

       let cop_loc_names = ["56-and-park",
                            "harper-and-harper-ct",
                            "57-and-harper-ct",
                            "56-and-everett(west)",
                            "54-and-cornell"]
           robber_loc_name = "56-and-harper(west)"
           cop_states = map ((loc_to_state PT_cop_car) .
                             fromJust .
                             name_to_loc) cop_loc_names
           robber_state = loc_to_state PT_robber $
                          fromJust $
                          name_to_loc  robber_loc_name
           ss = SearchState {
                    ss_epoch = 46,
                    ss_cop_states = map Just cop_states,
                    ss_robber_state = robber_state,
                    ss_last_smell_epoch = 0 }

           plugins = robber_plugins_init ws

           locs_up_to_distance_cached =
               rpi_cache_locs_up_to_distance (rpi_cache plugins)
           locs_up_to_distance_same_state_cached =
               rpi_cache_locs_up_to_distance_same_state (rpi_cache plugins)

           opts = make_minimax_opts ws
                  locs_up_to_distance_cached
                  locs_up_to_distance_same_state_cached []
       minimax_g <- newStdGen
       let search_moves = minimax opts minimax_g ss

       --putStrLn $ show $ take 8 search_moves

       let legal_moves_cached =
               rpi_cache_legal_moves (rpi_cache plugins)
           legal_anti_moves_cached =
               rpi_cache_legal_anti_moves (rpi_cache plugins)

       let tracking0 = tracking_init ws
           test_next_tracking tr =
               let tr' =
                     tracking_advance legal_moves_cached tr
                   tr'' =
                     tracking_update_possible
                     legal_moves_cached legal_anti_moves_cached 0
                     (S.singleton (loc_to_state PT_robber
                                   (ws_robber_start ws))) tr'
               in  tr''
           tracking_list = iterate test_next_tracking tracking0
           tracking_show_list =
               map (show . (map S.size) . elems) tracking_list
       putStr $ unlines tracking_show_list

{-
       putStrLn $ unlines $
                map (show . ((map ps_to_str) . S.toList)) states_at_distance
-}
       putStrLn "Goodbye, world!"
